package com.ssiisample.imageProcessingSample;

import java.io.File;
import java.io.FileOutputStream;

import android.app.Activity;
import android.content.ActivityNotFoundException;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.Bitmap.Config;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore.Images.Media;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;

public class ImageProcessingSample extends Activity {
	int mPreviewWidth=176,mPreviewHeight=144;		//カメラのプレビューサイズ
	float mScaleWidth, mScaleHeight;				//画面サイズとプレビューサイズの比率
	int mIntImage[]=null;							//ARGB8888の画像格納用
	ColorConvert mColorConvert=new ColorConvert();	//色変換用クラス
	ColorConvert.YUVtoRGB mYUV2RGB;

	
	CameraCallback mCameraCallback=null;
	
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		//setContentView(R.layout.main);//元々の表示画面をコメントアウト
		//画面の向きを横で固定
		this.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
		//全画面表示
		this.getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
		//スリープ無効
		this.getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
		//タイトルの非表示
		this.requestWindowFeature(Window.FEATURE_NO_TITLE);
		//レイアウト設定
		final ViewGroup.LayoutParams layoutParams = new ViewGroup.LayoutParams(
				ViewGroup.LayoutParams.FILL_PARENT,
				ViewGroup.LayoutParams.FILL_PARENT);

		//カメラ映像の表示
		mCameraCallback=new CameraCallback(this);
		setContentView(mCameraCallback, layoutParams);//CameraCallbackを描画画面に設定
		//独自プレビュー描画画面
		addContentView(new GraphicsView(this), layoutParams);

	}

	//独自プレビュー画面
	private class GraphicsView extends View{
		Paint paint;
		public GraphicsView(Context c){
			super(c);
			paint=new Paint();
		}
		@Override
		protected void onDraw(Canvas canvas){
			if(mIntImage==null)return;
			canvas.drawRGB(128, 128, 128);//SurfaceViewのカメラプレビューは使わないので塗りつぶしておく
			//canvas.scale(mScaleWidth, mScaleHeight);
			canvas.scale(mScaleHeight, mScaleHeight);//画面の高さにあわせて拡大
			canvas.drawBitmap(mIntImage, 0, mPreviewWidth, 0, 0, mPreviewWidth, mPreviewHeight, false, paint);//独自プレビューの描画
			//invalidate();
		}

	}

	//画面にタッチされた際の処理
	public boolean onTouchEvent(MotionEvent event) {
	    switch (event.getAction()) {
	    case MotionEvent.ACTION_DOWN:
	        break;
	    case MotionEvent.ACTION_UP:
	        //shareImage();
	        mCameraCallback.autoFocus();
	        break;
	    case MotionEvent.ACTION_MOVE:
	        break;
	    case MotionEvent.ACTION_CANCEL:
	        break;
	    }	    
	    return super.onTouchEvent(event);
	}

	//初期化用
	public void initImageData(int preview_width, int preview_height,int screen_width,int screen_height){
		Log.i("initImageData", "preview width:"+preview_width+" height:"+preview_height);
		mPreviewWidth=preview_width;
		mPreviewHeight=preview_height;
		mScaleWidth=(float)screen_width/mPreviewWidth;
		mScaleHeight=(float)screen_height/mPreviewHeight;
		mIntImage=new int[mPreviewWidth*mPreviewHeight];
		mYUV2RGB = mColorConvert.newYUVtoRGB(mPreviewWidth, mPreviewHeight, 2,3,3);


		//Intent連携処理
		Uri imageUri = null;
		try{imageUri=Uri.parse(getIntent().getExtras().get("android.intent.extra.STREAM").toString());}
		catch(Exception e){}
		if (imageUri != null) {
		    Log.v("Intent", "暗黙的intent起動");
		    Bitmap bmp = null;
		    try {bmp = Media.getBitmap(getContentResolver(), imageUri);}
		    catch (Exception e) {}
		    if (bmp != null) {
		    	bmp=Bitmap.createScaledBitmap(bmp, mPreviewWidth, mPreviewHeight, false);
		    	mIntBitmapData=new int[bmp.getWidth()*bmp.getHeight()];
		    	bmp.getPixels(mIntBitmapData, 0, bmp.getWidth(), 0, 0, bmp.getWidth(), bmp.getHeight());
		    }
		}
		else{
		    Log.v("Intent", "ランチャー起動");
		}

	}
	//画像処理
	public void cameraLoop(byte yuv[]){
		//3x3のフィルター処理
		//ImageFilter3x3(yuv,mPreviewWidth,mPreviewHeight,mFilter3x3_smoothing);
		//シンメトリー
		symmetryFilter(yuv,mPreviewWidth,mPreviewHeight);

		//YUVからRGBへ変換
		mYUV2RGB.getFast(yuv, mIntImage);
		ImageBlend();
	}

	//他アプリケーションに画像を送る
	void shareImage(){
		String path_str=Environment.getExternalStorageDirectory().toString()+"/"+this.getPackageName()+"/sample.jpg";
		Bitmap t_bmp=Bitmap.createBitmap(mIntImage, mPreviewWidth, mPreviewHeight, Config.RGB_565);
		try{
			//保存先フォルダの確認
			File outputDir = new File(Environment.getExternalStorageDirectory(), this.getPackageName());  
			if (outputDir.exists() == false)outputDir.mkdir();//なければ作る 
			
			//bitmapをJpegに変換してSDカードに書き出し
			FileOutputStream out=new FileOutputStream(path_str);
			t_bmp.compress(CompressFormat.JPEG, 100, out);
			out.flush();out.close();    
			
			//外部アプリケーションに渡す
			Intent intent = new Intent(Intent.ACTION_SEND);
			intent.setType("image/jpg");
			intent.putExtra(Intent.EXTRA_STREAM, Uri.fromFile(new File(path_str)));
			try {
				this.startActivity(Intent.createChooser(intent, null));
			} catch (ActivityNotFoundException e) {
			}			
		}catch(Exception e){}     
	}
	
	//別途用意した画像とプレビュー画像を合成
	int mIntBitmapData[]=null;
	void ImageBlend() {
		if(mIntBitmapData==null)return;
		for (int j = 0; j < mPreviewHeight; j++) {
			for (int i = 0; i < mPreviewWidth; i++) {
				int r1=(mIntBitmapData[i+j*mPreviewWidth]>>16)&0xff;
				int r2=(mIntImage[i+j*mPreviewWidth]>>16)&0xff;
				int g1=(mIntBitmapData[i+j*mPreviewWidth]>>8)&0xff;
				int g2=(mIntImage[i+j*mPreviewWidth]>>8)&0xff;
				int b1=(mIntBitmapData[i+j*mPreviewWidth])&0xff;
				int b2=(mIntImage[i+j*mPreviewWidth])&0xff;
				mIntImage[i+j*mPreviewWidth]=(((r1+r2)/2)<<16)|(((g1+g2)/2)<<8)|((b1+b2)/2)|0xff000000;
			}
		}
	}

	int mFilter3x3_sharpness[] = { -1, -1, -1, -1, 9, -1, -1, -1, -1, 1 };
	int mFilter3x3_smoothing[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 9 };
	byte mBuffer[]=null;//画像の一時保存用
	//3x3フィルター
	void ImageFilter3x3(byte[] aSrc,int width,int height, int[] filter) {
		if(mBuffer==null)mBuffer=new byte[width * height];
		int pixPtr;
		for (int j = 1; j < height - 1; j++) {
			pixPtr = j * width + 1;
			for (int i = 1; i < width - 1; i++) {
				int v = ((aSrc[pixPtr - 1 - width] & 0xff) * filter[0]
						+ (aSrc[pixPtr - width] & 0xff) * filter[1]
						+ (aSrc[pixPtr + 1 - width] & 0xff) * filter[2]
						+ (aSrc[pixPtr - 1] & 0xff) * filter[3]
						+ (aSrc[pixPtr] & 0xff) * filter[4]
						+ (aSrc[pixPtr + 1] & 0xff) * filter[5]
						+ (aSrc[pixPtr - 1 +width] & 0xff) * filter[6]
						+ (aSrc[pixPtr + width] & 0xff) * filter[7] + (aSrc[pixPtr
						+ 1 + width] & 0xff)
						* filter[8])
						/ filter[9];
				v=(v>255)?255:((v<0)?0:v);
				mBuffer[pixPtr] = (byte) v;
				pixPtr++;
			}
		}
		System.arraycopy(mBuffer, 0, aSrc, 0, width * height);
	}
	//シンメトリー
	void symmetryFilter(byte[] aSrc,int width,int height){
		int size=width * height;
		int h_width=width/2;
		if(mBuffer==null)mBuffer=new byte[size*3/2];
		int pixPtr,cPtr;
		for (int j = 0; j < height; j++) {
			pixPtr = j * width;
			cPtr=(j*width>>1)+size;
			//for (int i = 0; i < width/2; i++) {
			for (int i = 0; i < width; i+=2) {
				//輝度
				mBuffer[pixPtr] = aSrc[pixPtr];
				mBuffer[pixPtr+width/2] = aSrc[pixPtr+h_width-i];
				pixPtr++;
				//色差
				//mBuffer[((i+width/2)&0xfffe)+0+(j>>1)*width+size]=aSrc[((width/2-i)&0xfffe)+0+(j>>1)*width+size];
				//mBuffer[((i+width/2)&0xfffe)+1+(j>>1)*width+size]=aSrc[((width/2-i)&0xfffe)+1+(j>>1)*width+size];
				if( (j&1)!=0 )continue;
				mBuffer[cPtr]=aSrc[cPtr];
				mBuffer[cPtr+h_width]=aSrc[cPtr+h_width-i];
				cPtr++;

			}
		}
		System.arraycopy(mBuffer, 0, aSrc, 0, size*3/2);
	}












}
