package com.ssiisample.imageProcessingSample2;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

import javax.microedition.khronos.egl.EGL10;
import javax.microedition.khronos.egl.EGL11;
import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.egl.EGLContext;
import javax.microedition.khronos.egl.EGLDisplay;
import javax.microedition.khronos.egl.EGLSurface;
import javax.microedition.khronos.opengles.GL10;

import android.app.Activity;
import android.content.Context;
import android.graphics.PixelFormat;
import android.opengl.GLU;
import android.util.AttributeSet;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public class OverlayGLSurfaceView extends SurfaceView implements  SurfaceHolder.Callback{
	protected EGLContext eglContext;
	protected SurfaceHolder sHolder;

	int mTextureSize=256;

	int init_flag=0;
	int mWidth;
	int mHeight;
	EGLSurface surface;
	EGLDisplay dpy;
	EGL10 egl;
	GL10 gl;
	FloatBuffer cubeBuff;
	FloatBuffer texBuff;


	public byte[] mUShortData=new byte[mTextureSize*mTextureSize*2];


	int[] cameraTexture=new int[1];

	public void setTextureSize(int aSize){
		if(aSize<256)aSize=256;
		else if(aSize<512)aSize=512;
		else if(aSize<1024)aSize=1024;
		else aSize=2048;
		mTextureSize=aSize;
		mUShortData=new byte[mTextureSize*mTextureSize*2];


	}



	final static float camObjCoord[] = new float[] {
		-1.875f, -1.33333f,  0f,
		1.875f, -1.33333f,  0f,
		-1.875f,  1.33333f,  0f,
		1.875f,  1.33333f,  0f
	};
	final static float camTexCoords[] = new float[] {
		// Camera preview
		 0.0f, 0.625f,
		 0.9375f, 0.625f,
		 0.0f, 0.0f,
		 0.9375f, 0.0f
	};

	public OverlayGLSurfaceView(Context c,AttributeSet attrs) {
		super(c,attrs);

		cubeBuff = makeFloatBuffer(camObjCoord);
		texBuff = makeFloatBuffer(camTexCoords);
		setSurfaceHolder();

	}
	public OverlayGLSurfaceView(Context c) {
		super(c);
		cubeBuff = makeFloatBuffer(camObjCoord);
		texBuff = makeFloatBuffer(camTexCoords);
		setSurfaceHolder();
	}
	public void setSurfaceHolder(){
		sHolder = getHolder();
		sHolder.setFormat(PixelFormat.TRANSLUCENT);
		sHolder.addCallback(this);
		sHolder.setType(SurfaceHolder.SURFACE_TYPE_GPU);
	}

	FloatBuffer makeFloatBuffer(float[] arr) {
		ByteBuffer bb = ByteBuffer.allocateDirect(arr.length*4);
		bb.order(ByteOrder.nativeOrder());
		FloatBuffer fb = bb.asFloatBuffer();
		fb.put(arr);
		fb.position(0);

		return fb;
	}
	public void surfaceChanged(SurfaceHolder holder, int format, int width,	int height) {
		Log.e("GL","4");
		synchronized (this) {
			mWidth = width;
			mHeight = height;
		}
		init();
		Log.e("GL","4");
	}
	public void surfaceCreated(SurfaceHolder holder) {
		Log.e("GL","5");

	}
	public void surfaceDestroyed(SurfaceHolder holder) {
		release();
    }
	public void release(){
		init_flag=0;
		Log.e("GL","6");
	    egl.eglMakeCurrent(dpy, EGL10.EGL_NO_SURFACE, EGL10.EGL_NO_SURFACE, EGL10.EGL_NO_CONTEXT);
	    egl.eglDestroySurface(dpy, surface);
	    egl.eglDestroyContext(dpy, eglContext);
	    egl.eglTerminate(dpy);
	    Log.e("GL","6");
	}
	protected void init() {
		egl = (EGL10)EGLContext.getEGL();
		dpy = egl.eglGetDisplay(EGL10.EGL_DEFAULT_DISPLAY);

		int[] version = new int[2];
        egl.eglInitialize(dpy, version);

        int[] configSpec = {
                EGL10.EGL_RED_SIZE,      5,
                EGL10.EGL_GREEN_SIZE,    6,
                EGL10.EGL_BLUE_SIZE,     5,
                EGL10.EGL_DEPTH_SIZE,    8,
                EGL10.EGL_ALPHA_SIZE,    8,
                EGL10.EGL_NONE
        };

        EGLConfig[] configs = new EGLConfig[1];
        int[] num_config = new int[1];
        egl.eglChooseConfig(dpy, configSpec, configs, 1, num_config);
        EGLConfig config = configs[0];

		eglContext = egl.eglCreateContext(dpy, config, EGL10.EGL_NO_CONTEXT, null);

		surface = egl.eglCreateWindowSurface(dpy, config, sHolder, null);
		egl.eglMakeCurrent(dpy, surface, surface, eglContext);

		gl = (GL10)eglContext.getGL();

		gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
		gl.glClearDepthf(1.0f);

		gl.glVertexPointer(3, GL10.GL_FLOAT, 0, cubeBuff);
		gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
		gl.glTexCoordPointer(2, GL10.GL_FLOAT, 0, texBuff);
		gl.glEnableClientState(GL10.GL_TEXTURE_COORD_ARRAY);
		gl.glEnable(GL10.GL_TEXTURE_2D);

		gl.glMatrixMode(GL10.GL_PROJECTION);
		gl.glLoadIdentity();
		gl.glViewport(0,0,mWidth,mHeight);
		GLU.gluPerspective(gl, 45.0f, (float)1.4, 1f, 100f);

		gl.glMatrixMode(GL10.GL_MODELVIEW);
		gl.glLoadIdentity();
		GLU.gluLookAt(gl, 0, 0, 5.1f, 0, 0, 0, 0, 1, 0);
		gl.glNormal3f(0,0,1);

		init_flag=1;
	}

	public void draw(float rotate_xy, float rotate_xz) {
		if (init_flag == 0) {
			return;
		}

		gl.glClear(GL10.GL_COLOR_BUFFER_BIT | GL10.GL_DEPTH_BUFFER_BIT);
		gl.glPushMatrix();
		gl.glTranslatef(0f, 0f, 1.875f);
		gl.glRotatef(-rotate_xz, 1, 0, 0);
		gl.glRotatef(-rotate_xy, 0, 0, 1); // Rotate

		gl.glGenTextures(1, cameraTexture, 0);
		gl.glBindTexture(GL10.GL_TEXTURE_2D, cameraTexture[0]);
		gl.glTexImage2D(GL10.GL_TEXTURE_2D, 0, GL10.GL_RGB, mTextureSize,
				mTextureSize, 0, GL10.GL_RGB, GL10.GL_UNSIGNED_SHORT_5_6_5,
				ByteBuffer.wrap(mUShortData));
		gl.glTexParameterf(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_MIN_FILTER,
				GL10.GL_LINEAR);

		gl.glDrawArrays(GL10.GL_TRIANGLE_STRIP, 0, 4);

		egl.eglSwapBuffers(dpy, surface);
		gl.glPopMatrix();
		gl.glDeleteTextures(1, cameraTexture, 0);

		if (egl.eglGetError() == EGL11.EGL_CONTEXT_LOST) {
			Context c = getContext();
			if (c instanceof Activity) {
				((Activity) c).finish();
			}
		}
	}



}