package com.ssiisample.imageProcessingSample3;

import android.app.Activity;
import android.content.Context;
import android.content.pm.ActivityInfo;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.PointF;
import android.media.FaceDetector;
import android.media.FaceDetector.Face;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;

public class ImageProcessing extends Activity {
	FrameRateChecker mFPS = new FrameRateChecker(10);

	CameraCallback mCameraCallback = null; 
	GraphicsView mGraphicsView = null;

	int[] mIntImage = null;
	byte[] mYBuffer = null;
	int mWidth = 240, mHeight = 160;
	int mYStep = 2, mCrStep = 3, mCbStep = 3; 
	int mFrameCount = 0; 

	float mScaleWidth = 1, mScaleHeight = 1;

	int mSearchFlag=0;
	
	
	int[] mClippingIntData;
	FaceDetector.Face[] mFaces = new FaceDetector.Face[1];
	FaceDetector mFaceDetector;
	int mNumFaces = 0;
	int mSearchX = 120, mSearchY = 80;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		this.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
		this.getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
		this.getWindow().addFlags(
				WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON); 
		this.requestWindowFeature(Window.FEATURE_NO_TITLE);

		final ViewGroup.LayoutParams layoutParams = new ViewGroup.LayoutParams(
				ViewGroup.LayoutParams.FILL_PARENT,
				ViewGroup.LayoutParams.FILL_PARENT);

		mCameraCallback = new CameraCallback(this);
		// View
		mGraphicsView = new GraphicsView(this);
		setContentView(mCameraCallback, layoutParams);
		addContentView(mGraphicsView, layoutParams);

		addContentView(new StatusView(this), layoutParams);

	}

	public class StatusView extends View {
		Paint paint;
		String mStatusStr[] = { "", "", "", "", "" };

		public StatusView(Context context) {
			super(context);
			paint = new Paint();
		}
		
		Matrix init_matrix = new Matrix();
		float face_zoom = (float) 0.001;
		private void drawFaceImage(Canvas canvas){
			
			paint.setColor(Color.argb(255, 255, 255, 255));

			canvas.scale(mScaleWidth, mScaleHeight);

				canvas.drawLines(new float[] { (mWidth >> 1) - 8, mHeight >> 1,
						(mWidth >> 1) + 8, mHeight >> 1, mWidth >> 1,
						(mHeight >> 1) - 8, mWidth >> 1, (mHeight >> 1) + 8 },
						paint);

				canvas.drawLines(new float[] { 	mSearchX - 16, mSearchY - 16,	mSearchX - 16, mSearchY + 16,
												mSearchX - 16, mSearchY + 16,	mSearchX + 16, mSearchY + 16,
												mSearchX + 16, mSearchY + 16,	mSearchX + 16, mSearchY - 16,
												mSearchX + 16, mSearchY - 16,	mSearchX - 16, mSearchY - 16 }, paint);
				for (int i = 0; i < mNumFaces; i++) { //顔検出の結果を描画
					Face face = mFaces[i];
					PointF midPoint = new PointF(0, 0);
					face.getMidPoint(midPoint);
					float eyesDistance = face.eyesDistance();
					face_zoom = eyesDistance * 0.020f;
					paint.setStyle(Paint.Style.STROKE);
					if (eyesDistance == 20)
						paint.setColor(Color.argb(255, 255, 0, 0));
					else
						paint.setColor(Color.argb(255, 0, 255, 0));
					canvas.drawCircle(midPoint.x + mSearchX - 40,(midPoint.y + mSearchY - 50),(float) (eyesDistance * 1.5), paint);
				}

			canvas.setMatrix(init_matrix);

			if (mClippingIntData != null)
				canvas.drawBitmap(mClippingIntData, 0, 80,	0, 48, 80, 100, true, paint);//切り出した画像の描画

	}


		@Override
		protected void onDraw(Canvas canvas) {
			drawFaceImage(canvas);
			if (mFrameCount % 5 == 0) {
				mStatusStr[0] = mFPS.getString() + "fps ImageProcessing";
				if (mCameraCallback != null)
					mStatusStr[1] = mCameraCallback.mFPS.getString()
							+ "fps CameraPreview";

				mStatusStr[2] = mGraphicsView.mFPS.getString() + "fps View";

				mStatusStr[4] = "width:" + mWidth + " height:" + mHeight;
			}
			paint.setColor(Color.rgb(255, 255, 255));
			canvas.drawText(mStatusStr[0], 10, 10, paint);
			canvas.drawText(mStatusStr[1], 10, 10 + 12, paint);
			canvas.drawText(mStatusStr[2], 10, 10 + 12 * 2, paint);
			canvas.drawText(mStatusStr[3], 256, 10, paint);
			canvas.drawText(mStatusStr[4], 256, 10 + 12, paint);
		}
	}

	//画像処理結果の描画
	private class GraphicsView extends View {
		private FrameRateChecker mFPS = new FrameRateChecker(10);
		Paint paint;

		public GraphicsView(Context context) {
			super(context);
			paint = new Paint();
		}

		@Override
		protected void onDraw(Canvas canvas) {
			this.mFPS.measurement();
			canvas.drawRGB(128, 128, 128);//SurfaceViewのカメラプレビューは使わないので塗りつぶしておく
			canvas.scale(mScaleWidth, mScaleHeight);//画面サイズに拡大
			canvas.drawBitmap(mIntImage, 0, mWidth, 0, 0, mWidth, mHeight,false, paint);//独自プレビューの描画
		}

	}
	
	//キーイベント
	public boolean dispatchKeyEvent(KeyEvent event) {
		if (event.getAction() == KeyEvent.ACTION_UP) {
			switch (event.getKeyCode()) {
			case KeyEvent.KEYCODE_DPAD_CENTER:
			case KeyEvent.KEYCODE_SEARCH:
				mSearchFlag = 1;//決定キーからサーチキーで探索位置のリセット
				return true;
			default:
			}
		}
		return super.dispatchKeyEvent(event);
	}
	//タッチイベント
	public boolean onTouchEvent(MotionEvent event) {
		switch (event.getAction()) {
		case MotionEvent.ACTION_DOWN:
			break;
		case MotionEvent.ACTION_UP:
			mSearchX=(int)(event.getX()/mScaleWidth);
			mSearchY=(int)(event.getY()/mScaleHeight);
			mSearchFlag = 2;
			break;
		case MotionEvent.ACTION_MOVE:
			break;
		case MotionEvent.ACTION_CANCEL:
			break;
	    }
	    return super.onTouchEvent(event);
	}
	//画像の初期化
	public void initImageData(int cam_w, int cam_h, int scr_w, int scr_h) {
		mIntImage = new int[cam_w * cam_h];
		mYBuffer = new byte[cam_w * cam_h];
		mWidth = cam_w;
		mHeight = cam_h;

		mScaleWidth = (float) scr_w / cam_w;
		mScaleHeight = (float) scr_h / cam_h;
		
		mClippingIntData = new int[(80) * (120)];
		mFaceDetector = new FaceDetector(80, 100, mFaces.length);
		
		initTable(cam_w, cam_h, mYStep, mCrStep, mCbStep);
	}
	//ネイティブで確保したテーブルの開放
	public void releaseImageData(){
		releaseTable();
	}
	//画像処理
	void cameraLoop(byte yuv_data[]) {
		mFrameCount++;
		mFPS.measurement();		

		if(mSearchFlag==2){//タッチイベントがあった場合はタッチ位置を探索位置に
			setNextSearchPos(yuv_data, (int) (mSearchX ), (int) (mSearchY));
			mSearchFlag=0;
		}
		

		int search_pos=convertYUV2RGB8888(yuv_data, mIntImage, mSearchFlag);//YUV420->ARGB8888
		clippingRGB(yuv_data, mClippingIntData, search_pos);
		mNumFaces = mFaceDetector.findFaces(Bitmap.createBitmap(mClippingIntData, 80,100, Bitmap.Config.RGB_565), mFaces); //切り抜いた領域に対して顔検出
		mSearchFlag=0;
		mSearchY = search_pos % 1000;
		mSearchX = search_pos / 1000;
		for (int i = 0; i < mNumFaces; i++) { //検出した顔領域の位置を次STEPでの探索位置にセット
			Face face = mFaces[i];
			PointF midPoint = new PointF(0, 0);
			face.getMidPoint(midPoint); //顔の位置

			setNextSearchPos(yuv_data, (int) (midPoint.x
					+ mSearchX - 40), (int) (midPoint.y
							+ mSearchY - 50));
		}

	}


	static {
		System.loadLibrary("image_processing");
	}

	private native void initTable(int widht, int height, int aY, int aCr,int aCb);
	private native void releaseTable();
	private native int convertYUV2RGB8888(byte[] src, int[] dst, int flag);
	private native void clippingRGB(byte[] src, int[] dst, int pos);
	private native void setNextSearchPos(byte[] src, int x, int y);



}