package com.ssiisample.imageProcessingSample3;

import java.lang.reflect.Method;
import java.util.List;

import android.graphics.PixelFormat;
import android.hardware.Camera;
import android.hardware.Camera.Size;
import android.util.Log;

public class RefCamera{

	private static Method ref_getSupportedPreviewSizes=null;
	private static Method ref_addCallbackBuffer=null;
	private static Method ref_setPreviewCallbackWithBuffer=null;

	static {
		initCompatibility();
	};
	private static void initCompatibility() {
		try {
			//プレビューサイズの設定
			ref_getSupportedPreviewSizes = Camera.Parameters.class.getMethod("getSupportedPreviewSizes", new Class[]{} );
			//setPreviewCallbackWithBuffer関連
			ref_addCallbackBuffer = Camera.class.getMethod("addCallbackBuffer", byte[].class);
			ref_setPreviewCallbackWithBuffer= Camera.class.getMethod("setPreviewCallbackWithBuffer", Camera.PreviewCallback.class );

		} catch (NoSuchMethodException nsme) {}
	}
	//reflect
	@SuppressWarnings("unchecked")
	public static List<Size> getSupportedPreviewSizes(Camera.Parameters parameters){
		try {
			if(ref_getSupportedPreviewSizes!=null)
				return (List<Size>)ref_getSupportedPreviewSizes.invoke(parameters);
			else{
				return null;
			}
		} catch (Exception e){return null;}
	}
	public static void addCallbackBuffer(Camera camera,byte[] buffer){
		try {
			if(ref_addCallbackBuffer!=null){
				ref_addCallbackBuffer.invoke(camera,buffer);
				return;
			}
		} catch (Exception e){}
	}
	public static void setPreviewCallbackWithBuffer(Camera camera,Camera.PreviewCallback cb){
		try {
			if(ref_setPreviewCallbackWithBuffer!=null){
				ref_setPreviewCallbackWithBuffer.invoke(camera,cb);
				return;
			}
		} catch (Exception e){}
	}

	//setPreviewCallbackWithBuffer関連
	//プレビューサイズでのバッファ初期化＋addCallbackBuffer
	public static void setCallbackBuffer(Camera camera){
		PixelFormat pixel_info=new PixelFormat();
		PixelFormat.getPixelFormatInfo(camera.getParameters().getPreviewFormat(),pixel_info);
		byte buffer[]=new byte [camera.getParameters().getPreviewSize().width*
		                        camera.getParameters().getPreviewSize().height*
		                        pixel_info.bitsPerPixel/8];//プレビューサイズでバッファを確保
		addCallbackBuffer(camera,buffer);
	}
	//setPreviewCallbackWithBufferが利用できるか
	public static boolean isEnablePreviewWithBuffer(){
		if(ref_setPreviewCallbackWithBuffer!=null){
			return true;
		}
		return false;
	}


	//プレビューサイズの設定
	//パラメータ番号でプレビューサイズを設定
	public static Size setSupportedPreviewSize(Camera camera,int id){
		Camera.Parameters parameters = camera.getParameters();
		String size_str[]=getSupportedPreviewSizeString(camera,parameters);
		String size_num_str[]=size_str[id%size_str.length].split("x");
		Log.i("setPreviewSize","w:"+Integer.valueOf(size_num_str[0])+"h:"+Integer.valueOf(size_num_str[1]));
		try{
			parameters.setPreviewSize(Integer.valueOf(size_num_str[0]),Integer.valueOf(size_num_str[1]));
			camera.setParameters(parameters);
		}catch(Exception e){
			parameters.setPreviewSize(320,240);//例外が発生したらQVGAにしてみる
			camera.setParameters(parameters);
		}
		return camera.getParameters().getPreviewSize();
	}
	//指定サイズに最も近い値にプレビューサイズを設定
	public static Size setNearestPreviewSize(Camera camera,int w,int h){
		Camera.Parameters parameters = camera.getParameters();
		String size_str[]=getSupportedPreviewSizeString(camera,parameters);

		int mini_value=Integer.MAX_VALUE;
		int id=0;
		for(int i=0;i<size_str.length;i++){
			String size_num_str[]=size_str[i].split("x");
			int ws=w-Integer.valueOf(size_num_str[0]);
			int hs=h-Integer.valueOf(size_num_str[1]);
			if(mini_value>ws*ws+hs*hs){
				mini_value=ws*ws+hs*hs;
				id=i;
			}
		}
		if(mini_value!=0)Log.w("setNearestPreviewSize","'"+w+"x"+h +"'"+" not found.");
		return setSupportedPreviewSize(camera,id);
	}
	//設定可能なプレビューサイズを取得
	public static String[] getSupportedPreviewSizeString(Camera camera,Camera.Parameters parameters) {
		String[] size_str=null;
		List<Size> supportedSizes=getSupportedPreviewSizes(parameters);
		if(supportedSizes!=null){
			Log.i("getPreviewSize","getSupportedPreviewSizes");
			size_str=new String[supportedSizes.size()];
			for(int i=0;i<supportedSizes.size();i++){
				size_str[i]=supportedSizes.get(i).width+"x"+supportedSizes.get(i).height;
			}
		}else{//flatten
			Log.i("getPreviewSize","parameters.flatten()");
			String flatten_str_t[]=parameters.flatten().split(";");
			String flatten_str[][]=new String[flatten_str_t.length][];
			for(int i=0;i<flatten_str_t.length;i++){
				flatten_str[i]=flatten_str_t[i].split("=");
				if(flatten_str[i].length>1){
					if(flatten_str[i][0].equals("preview-size-values")){
						size_str=flatten_str[i][1].split(",");
						break;
					}
				}
			}
			if(size_str==null){//プレビューサイズが取れなければHT-03Aと仮定
				Log.w("getPreviewSize","HT-03A?");
				//手探りで調べたHT-03Aで使えるサイズ
				size_str =new String[7];
				size_str[0]="176x144";
				size_str[1]="240x160";
				size_str[2]="320x240";
				size_str[3]="352x288";
				size_str[4]="384x288";
				size_str[5]="480x320";
				size_str[6]="640x480";
			}
		}
		for(int i=0;i<size_str.length;i++){
			Log.i("getPreviewSize","preview size "+i+":"+size_str[i]);
		}
		return size_str;
	}
}